<?php

namespace App\Http\Controllers\Admin\Dashboard;

use App\Filters\Application\Dashboard\DashboardPostOverviewFilter;
use App\Http\Controllers\Controller;
use App\Models\Application\Post\CommentReply;
use App\Models\Application\Post\Post;
use App\Models\Application\Post\PostComment;
use App\Models\Application\Post\PostReaction;
use App\Models\Core\Auth\User;
use App\Models\Core\Status;
use App\Services\Application\Dashboard\DashboardService;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;
use JetBrains\PhpStorm\ArrayShape;

class AppDashboardController extends Controller
{
    public $status;

    public function __construct()
    {
        $this->status = $this->getUserStatus();
    }

    #[ArrayShape(['total_user' => "int", 'total_active' => "int", 'total_de_active' => "int", 'total_invited' => "int"])] public function userCard(): array
    {
        $totalUser = $this->userCount();
        $totalActive = $this->userCount($this->status['status_active']);
        $totalDeactive = $this->userCount($this->status['status_inactive']);
        $totalInvited = $this->userCount($this->status['status_invited']);

        return [
            'total_user' => $totalUser,
            'total_active' => $totalActive,
            'total_de_active' => $totalDeactive,
            'total_invited' => $totalInvited
        ];
    }

    private function userCount($status = null): int
    {
        return User::query()
            ->when($status, fn(Builder $builder) => $builder->where('status_id', $status))
            ->count();
    }

    private function getUserStatus()
    {
        return Status::where('type', 'user')
            ->pluck('id', 'name');
    }

    public function postCard(): array
    {
        $totalPost = Post::query()->count();
        $totalReaction = PostReaction::query()->count();
        $totalComment = PostComment::query()->count();
        $totalReply = CommentReply::query()->count();

        return [
            'total_post' => $totalPost,
            'total_reaction' => $totalReaction,
            'total_comment' => $totalComment,
            'total_reply' => $totalReply
        ];
    }

    public function topPoster(): array
    {
        $topThreePosters = Post::query()->groupBy('user_id')
            ->select('user_id', DB::raw('count(*) as posts'))
            ->orderBy('posts', 'DESC')
            ->with('user:id,first_name,last_name,uuid')
            ->limit(3)
            ->get();

        $threePosterName = [];
        $threePosterDeals = [];
        foreach ($topThreePosters as $value) {
            $threePosterName[] = $value->user->full_name;
            $threePosterDeals[] = $value->posts;
        }

        return [
            'label' => $threePosterName,
            'posts' => $threePosterDeals,
        ];
    }

    public function reactions() {
        $reactionsQuery = PostReaction::query()
            ->groupBy('reactable_type')
            ->select('reactable_type', DB::raw('count(*) as reaction_count'));

        if(!auth()->user()->isAdmin())
            $reactionsQuery->where('user_id', auth()->id());

        $reactions = $reactionsQuery->get();
        $response = [
            'post' => $reactions->where('reactable_type', Post::class)->sum('reaction_count'),
            'comment' => $reactions->where('reactable_type', PostComment::class)->sum('reaction_count'),
            'reply' => $reactions->where('reactable_type', CommentReply::class)->sum('reaction_count'),
            'total' => $reactions->sum('reaction_count'),
        ];
        $response['dataset'] = [$response['post'], $response['comment'], $response['reply']];
        return $response;
    }

    public function postOverview(): array
    {
        $filter = resolve(DashboardPostOverviewFilter::class);
        $postQuery = Post::filters($filter);

        if(!auth()->user()->isAdmin())
            $postQuery->where('user_id', auth()->id());

        $posts = $postQuery
                    ->get();

        $service = resolve(DashboardService::class);
        return $service->postOverview($posts);
    }

}
