<?php

namespace App\Http\Controllers\Application\Post;

use App\Events\NewNotification;
use App\Helpers\Traits\ApplicationFunction;
use App\Http\Controllers\Controller;
use App\Http\Requests\Application\Post\CommentReplyListRequest;
use App\Http\Requests\Application\Post\CommentReplyRequest;
use App\Models\Application\Post\CommentReply;
use App\Models\Application\Post\PostComment;
use App\Services\Application\Post\CommentReplyService;
use Illuminate\Http\Response;

class CommentReplyController extends Controller
{
    use ApplicationFunction;
    public function __construct(CommentReplyService $service)
    {
        $this->service = $service;
    }

    public function index(PostComment $post_comment, CommentReplyListRequest $request)
    {
        // ensure permission to access replies
        return $this->service
            ->with('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->withCount(['reactions'])
            ->where('comment_id', $post_comment->id)
            ->orderBy('created_at', 'asc')
            ->paginate(request()->get('per_page', 3));
    }

    public function store(PostComment $post_comment, CommentReplyRequest $request)
    {
        $attributes = array_merge($request
            ->only('comment'),
            [
                'comment_id' => $post_comment->id,
                'user_id' => auth()->id(),
            ]
        );
        $reply = $this->service
            ->setAttributes($attributes)
            ->save();

        $message = auth()->user()->full_name." replied to your comment ".$this->textShorter($post_comment->comment);
        broadcast(new NewNotification($post_comment->user_id, $message))->toOthers();

        $this->service->setModel($reply)->notify([$post_comment->user_id]);

        $reply->load('user:id,first_name,last_name,uuid',
            'user.profilePicture:path,fileable_type,fileable_id')
            ->loadCount(['reactions']);

        return created_responses('replies', ['reply' => $reply]);
    }

    public function show(PostComment $post_comment, CommentReply $comment_reply)
    {
        return $comment_reply
            ->load('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->loadCount(['reactions']);
    }

    public function update(PostComment $post_comment, PostComment $comment_reply, CommentReplyRequest $request)
    {
        // user can modify his own comment.
        if(auth()->id() != $comment_reply->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }

        $reply = $this->service
            ->setModel($comment_reply)
            ->setAttributes(
                $request->only('comment')
            )
            ->save();

        $reply->load('user:id,first_name,last_name,uuid',
            'user.profilePicture:path,fileable_type,fileable_id')
            ->loadCount(['reactions']);

        return updated_responses('replies', ['reply' => $reply]);
    }

    public function destroy(PostComment $post_comment, CommentReply $comment_reply)
    {
        // user can delete his own comment.
        if(auth()->id() != $comment_reply->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }

        $this->service
            ->setModel($comment_reply)
            ->delete();

        return deleted_responses('replies');
    }
}
