<?php

namespace App\Http\Controllers\Application\Post;

use App\Events\NewNotification;
use App\Helpers\Traits\ApplicationFunction;
use App\Http\Controllers\Controller;
use App\Http\Requests\Application\Post\PostCommentListRequest;
use App\Http\Requests\Application\Post\PostCommentRequest;
use App\Models\Application\Post\Post;
use App\Models\Application\Post\PostComment;
use App\Services\Application\Post\PostCommentService;
use Illuminate\Http\Response;

class PostCommentController extends Controller
{
    use ApplicationFunction;
    public function __construct(PostCommentService $service)
    {
        $this->service = $service;
    }

    public function index(Post $post, PostCommentListRequest $request)
    {
        // ensure permission to access comments
        return $this->service
            ->with('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->where('post_id', $post->id)
            ->withCount(['replies', 'reactions'])
            ->orderBy('created_at', 'asc')
            ->paginate(request()->get('per_page', 5));
    }

    public function store(Post $post, PostCommentRequest $request)
    {
        $attributes = array_merge($request
            ->only('comment'),
            [
                'post_id' => $post->id,
                'user_id' => auth()->id(),
            ]
        );
        $comment = $this->service
            ->setAttributes($attributes)
            ->save();

        $message = auth()->user()->full_name." commented to your post ".$this->textShorter($post->body);
        broadcast(new NewNotification($post->user_id, $message))->toOthers();

        $this->service->setModel($comment)->notify([$post->user_id]);

        $comment
            ->load('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->loadCount(['replies', 'reactions']);

        return created_responses('comment', ['comment' => $comment]);
    }

    public function show(Post $post, PostComment $post_comment)
    {
        return $post_comment
            ->load('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->loadCount(['replies', 'reactions']);
    }

    public function update(Post $post, PostComment $post_comment, PostCommentRequest $request)
    {
        // user can modify his own comment.
        if(auth()->id() != $post_comment->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }

        $comment = $this->service
            ->setModel($post_comment)
            ->setAttributes(
                $request->only('comment')
            )
            ->save();

        $comment
            ->load('user:id,first_name,last_name,uuid',
                'user.profilePicture:path,fileable_type,fileable_id',
            )
            ->loadCount(['replies', 'reactions']);

        return updated_responses('comment', ['comment' => $comment]);
    }

    public function destroy(Post $post, PostComment $post_comment)
    {
        // user can delete his own comment.
        if(auth()->id() != $post_comment->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }

        $this->service
            ->setModel($post_comment)
            ->deleteReplies()
            ->delete();

        return deleted_responses('comment');
    }
}
