<?php

namespace App\Http\Controllers\Application\Post;

use App\Filters\Application\Post\PostFilter;
use App\Http\Controllers\Controller;
use App\Http\Requests\Application\Post\PostRequest;
use App\Models\Application\Post\Post;
use App\Repositories\Core\Status\StatusRepository;
use App\Services\Application\Post\PostService;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class PostController extends Controller
{
    public function __construct(PostService $service, PostFilter $filter)
    {
        $this->service = $service;
        $this->filter = $filter;
    }

    public function index()
    {
        abort(Response::HTTP_UNAUTHORIZED);
        return $this->service
            ->filters($this->filter)
            ->with('user:id,first_name,last_name,uuid',
                'status:id,name'
            )
            ->withCount(['comments', 'reactions', 'views'])
            ->orderBy('created_at', 'desc')
            ->paginate(request()->get('per_page', 10));
    }

    public function store(PostRequest $request)
    {
        DB::transaction(function () use ($request) {
            $status = resolve(StatusRepository::class)->postPublished();
            $attributes = array_merge($request
                ->only('body', 'shared_with', 'shared_post_id'),
                [
                    'status_id' => $status,
                    'user_id' => auth()->id(),
                ]
            );
            $this->service
                ->setModel(
                    $this->service
                        ->setAttributes($attributes)
                        ->save()
                )
                ->syncTeams($request->get('teams', []))
                ->saveAttachments(request('attachments', []))
                ->notify();
                // save attachments.
                // save teams.
        });

        $post = $this->service->getModel();
        $post->fresh();

        $post->load('user:id,first_name,last_name,uuid', 'user.profilePicture:path,fileable_type,fileable_id',
            'status:id,name',
            'attachments:path,fileable_type,fileable_id'
        )->loadCount(['comments', 'reactions', 'views']);

        return created_responses('post',
            [
                'post' => $post
            ]);
    }

    public function show(Post $post)
    {
        return $post
            ->load('user:id,first_name,last_name,uuid',
                'status:id,name',
                'teams:id,name',
                'attachments:id,path,fileable_type,fileable_id'
            )
            ->loadCount(['comments', 'reactions', 'views']);
    }

    public function update(Post $post, PostRequest $request)
    {
        // user can modify his own post.
        if(auth()->id() != $post->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }
        $post = $this->service
            ->setModel($post)
            ->setAttributes(
                $request->only('body', 'shared_with', 'shared_post_id')
            )
            ->save();
        // save attachments.
        // save teams.

        $this->service
            ->setModel($post)
            ->syncTeams($request->get('teams', []))
            ->saveAttachments(request('attachments', []))
            ->removeAttachments(request('remove_attachments', []));

        $post->fresh();
        $post->load('user:id,first_name,last_name,uuid', 'user.profilePicture:path,fileable_type,fileable_id',
            'status:id,name',
            'attachments:path,fileable_type,fileable_id'
        )->loadCount(['comments', 'reactions', 'views']);

        return updated_responses('post', ['post' => $post]);
    }

    public function destroy(Post $post)
    {
        // user can delete his own post.
        if(auth()->id() != $post->user_id) {
            abort(Response::HTTP_UNAUTHORIZED);
        }

        $this->service
            ->setModel($post)
            ->syncTeams([])
            ->saveAttachments([])
            ->delete();

        return deleted_responses('post');
    }
}
